@@ -124,7 +124,9 @@ static void
 doTranslateMessage(MSG *m)
 {
 	if ((m->message == WM_KEYDOWN) && 
-	    ((m->wParam == VK_BACK))) return;
+	    ((m->wParam == VK_BACK) ||
+	     (((m->wParam == VK_ADD) || (m->wParam == VK_SUBTRACT)) &&
+	      (GetKeyState(VK_SHIFT) & 0x8000)))) return;
 	if ((m->message == WM_SYSKEYDOWN) && (m->wParam == VK_F10))
 	{
 	    m->message = WM_KEYDOWN;
@@ -313,6 +315,8 @@ LONG NT_handleMsg(
                 case VK_F10:
                 case VK_F11:
 	        case VK_F12:
+	        case VK_ADD:
+	        case VK_SUBTRACT:
 		    QEvent(wineventq,window,message,wParam,lParam);
 		    break;
 		default:
@@ -570,6 +574,8 @@ WinEventToXEvent(
                     case VK_F10:     key=XK_F10;         break;
                     case VK_F11:     key=XK_F11;         break;
                     case VK_F12:     key=XK_F12;         break;
+		    case VK_ADD:     key=XK_KP_Add;      break;
+		    case VK_SUBTRACT:key=XK_KP_Subtract; break;
 		    default:         key=0;              break;
 		}
 		if (key == 0) {
