@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	command.c
  *----------------------------------------------------------------------*
- * $Id: command.c,v 1.257.2.2 2002/11/29 00:03:17 gcw Exp $
+ * $Id: command.c,v 1.278 2004/01/29 23:54:18 gcw Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (c) 1992      John Bovey, University of Kent at Canterbury <jdb@ukc.ac.uk>
@@ -27,6 +27,7 @@
  * Copyright (c) 1998      Alfredo K. Kojima <kojima@windowmaker.org>
  * Copyright (c) 2001      Marius Gedminas
  *				- Ctrl/Mod4+Tab works like Meta+Tab (options)
+ * Copyright (c) 2003      Rob McMullen <robm@flipturn.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -662,7 +663,11 @@ rxvt_cmd_getc(rxvt_t *r)
 #define TIMEOUT_USEC	5000
     fd_set          readfds;
     int             quick_timeout, select_res;
+    int             want_motion_time, want_keypress_time;
     struct timeval  value;
+#if defined(POINTER_BLANK) || defined(CURSOR_BLINK)
+    struct timeval  tp;
+#endif
     struct rxvt_hidden *h = r->h;
 
     if (h->cmdbuf_ptr < h->cmdbuf_endp)	/* characters already read in */
@@ -674,10 +679,44 @@ rxvt_cmd_getc(rxvt_t *r)
 	if (h->v_bufstr < h->v_bufptr)	/* output any pending chars */
 	    rxvt_tt_write(r, NULL, 0);
 
+#if defined(POINTER_BLANK) || defined(CURSOR_BLINK)
+	tp.tv_sec = 0;		/* presume == 0 implies time not yet retrieved */
+#endif
+#if defined(CURSOR_BLINK)
+	want_keypress_time = 0;
+#endif
+#if defined(POINTER_BLANK)
+	want_motion_time = 0;
+#endif
+
 	while (XPending(r->Xdisplay)) {	/* process pending X events */
 	    XEvent          xev;
 
 	    XNextEvent(r->Xdisplay, &xev);
+#if defined(CURSOR_BLINK)
+	    if ((r->Options & Opt_cursorBlink)
+		&& xev.type == KeyPress) {
+		if (h->hidden_cursor) {
+		    h->hidden_cursor = 0;
+		    h->want_refresh = 1;
+		}
+		want_keypress_time = 1;
+	    }
+#endif
+#if defined(POINTER_BLANK)
+	    if ((r->Options & Opt_pointerBlank)
+		&& (h->pointerBlankDelay > 0)) {
+		if (xev.type == MotionNotify
+		    || xev.type == ButtonPress
+		    || xev.type == ButtonRelease) {
+		    if (r->h->hidden_pointer)
+			rxvt_pointer_unblank(r);
+		    want_motion_time = 1;
+		}
+		if (xev.type == KeyPress && r->h->hidden_pointer == 0)
+		    rxvt_pointer_blank(r);
+	    }
+#endif
 #ifdef USE_XIM
 	    if (!XFilterEvent(&xev, xev.xany.window))
 		rxvt_process_x_event(r, &xev);
@@ -690,6 +729,22 @@ rxvt_cmd_getc(rxvt_t *r)
 		return *h->cmdbuf_ptr++;
 	}
 
+#if defined(CURSOR_BLINK)
+	if (want_keypress_time) {
+	    (void)gettimeofday(&tp, NULL);
+	    h->lastcursorchange.tv_sec = tp.tv_sec;
+	    h->lastcursorchange.tv_usec = tp.tv_usec;
+	}
+#endif
+#if defined(POINTER_BLANK)
+	if (want_motion_time) {
+	    if (!tp.tv_sec)
+		(void)gettimeofday(&tp, NULL);
+	    h->lastmotion.tv_sec = tp.tv_sec;
+	    h->lastmotion.tv_usec = tp.tv_usec;
+	}
+#endif
+
 /*
  * the command input buffer is empty and we have no pending X events
  */
@@ -707,6 +762,20 @@ rxvt_cmd_getc(rxvt_t *r)
 	    }
 	}
 #endif /* MOUSE_WHEEL && MOUSE_SLIP_WHEELING */
+#ifdef SELECTION_SCROLLING
+	if (h->pending_scroll_selection) {
+	    quick_timeout = 1;
+	    if (!h->scroll_selection_delay--
+		&& rxvt_scr_page(r, h->scroll_selection_dir,
+		    h->scroll_selection_lines)) {
+		rxvt_selection_extend(r, h->selection_save_x,
+		    h->selection_save_y, h->selection_save_state);
+		h->scroll_selection_delay = SCROLLBAR_CONTINUOUS_DELAY;
+		h->refresh_type |= SMOOTH_REFRESH;
+		h->want_refresh = 1;
+	    }
+	}
+#endif
 #ifndef NO_SCROLLBAR_BUTTON_CONTINUAL_SCROLLING
 	if (scrollbar_isUp() || scrollbar_isDn()) {
 	    quick_timeout = 1;
@@ -733,11 +802,71 @@ rxvt_cmd_getc(rxvt_t *r)
 	    quick_timeout |= h->want_full_refresh;
 #endif
 	}
+
+#if defined(POINTER_BLANK) || defined(CURSOR_BLINK)
+	{
+	    int             set_quick_timeout = 0;
+	    long            csdiff, psdiff;
+
+#define BLINK_TIME	500000L
+	    csdiff = psdiff = 60000000L;	/* or, say, LONG_MAX */
+# if defined(CURSOR_BLINK)
+	    if (r->Options & Opt_cursorBlink) {
+		if (!tp.tv_sec)	/* didn't get it before so get it now */
+		    (void)gettimeofday(&tp, NULL);
+
+		csdiff = (tp.tv_sec - h->lastcursorchange.tv_sec) * 1000000L
+			 + tp.tv_usec - h->lastcursorchange.tv_usec;
+		if (csdiff > BLINK_TIME) { /* XXX: settable blink times */
+		    h->lastcursorchange.tv_sec = tp.tv_sec;
+		    h->lastcursorchange.tv_usec = tp.tv_usec;
+		    h->hidden_cursor = !h->hidden_cursor;
+		    csdiff = 0;
+		} else
+		    csdiff = BLINK_TIME - csdiff;
+		set_quick_timeout = 1;
+	    }
+# endif
+# if defined(POINTER_BLANK)
+	/*
+	 * If we haven't moved the pointer for a while
+	 */
+	    if ((r->Options & Opt_pointerBlank)
+		&& (h->pointerBlankDelay > 0)
+		&& (h->hidden_pointer == 0)) {
+		long            pdelay;
+
+		if (!tp.tv_sec)	/* didn't get it before so get it now */
+		    (void)gettimeofday(&tp, NULL);
+		psdiff = (tp.tv_sec - h->lastmotion.tv_sec) * 1000000L
+			 + tp.tv_usec - h->lastmotion.tv_usec;
+		pdelay = h->pointerBlankDelay * 1000000L;
+		if (psdiff >= pdelay)
+		    rxvt_pointer_blank(r);
+		else {
+		    set_quick_timeout = 1;
+		    psdiff = pdelay - psdiff;
+		}
+	    }
+# endif
+	    if (!quick_timeout && set_quick_timeout) {
+		MIN_IT(csdiff, psdiff);
+		value.tv_sec =  csdiff / 1000000L;
+		value.tv_usec = csdiff % 1000000L;
+		quick_timeout = 1;
+	    }
+	}
+#endif
+
 	if ((select_res = select(r->num_fds, &readfds, NULL, NULL,
 				 (quick_timeout ? &value : NULL))) == 0) {
 	/* select statement timed out - we're not hard and fast scrolling */
 	    h->refresh_limit = 1;
 	}
+#if defined(CURSOR_BLINK)
+	if (r->Options & Opt_cursorBlink)
+	    h->want_refresh = 1;
+#endif
 
     /* See if we can read new data from the application */
 	if (select_res > 0 && FD_ISSET(r->cmd_fd, &readfds)) {
@@ -777,6 +906,35 @@ rxvt_cmd_getc(rxvt_t *r)
 }
 /*}}} */
 
+/* EXTPROTO */
+void
+rxvt_pointer_unblank(rxvt_t *r)
+{
+    XDefineCursor(r->Xdisplay, r->TermWin.vt, r->TermWin_cursor);
+    rxvt_recolour_cursor(r);
+#ifdef POINTER_BLANK
+    r->h->hidden_pointer = 0;
+    if (r->h->pointerBlankDelay > 0) {
+	struct timeval  tp;
+
+	(void)gettimeofday(&tp, NULL);
+	r->h->lastmotion.tv_sec = tp.tv_sec;
+	r->h->lastmotion.tv_usec = tp.tv_usec;
+    }
+#endif
+}
+
+#ifdef POINTER_BLANK
+/* INTPROTO */
+void
+rxvt_pointer_blank(rxvt_t *r)
+{
+    XDefineCursor(r->Xdisplay, r->TermWin.vt, r->h->pointer_blank);
+    XFlush(r->Xdisplay);
+    r->h->hidden_pointer = 1;
+}
+#endif
+
 /* INTPROTO */
 void
 rxvt_mouse_report(rxvt_t *r, const XButtonEvent *ev)
@@ -854,12 +1012,46 @@ rxvt_W11_process_x_event(XEvent *ev)
 }
 #endif
 
+
+/* INTPROTO */
+void
+rxvt_set_bg_focused(rxvt_t *r, Bool focus)
+{
+    XColor          xcol;
+    XGCValues       gcvalue;
+
+    if (!ISSET_PIXCOLOR(r->h, Color_ufbg))
+	return;
+    
+    gcvalue.background = focus ? r->h->color_origbg : r->PixColors[Color_ufbg];
+    r->PixColors[Color_bg] = gcvalue.background;
+
+#if defined(TRANSPARENT) || defined(XPM_BACKGROUND)
+    if (!focus
+	|| (!(r->Options & Opt_transparent) && r->TermWin.pixmap == None))
+#endif
+    {
+	XChangeGC(r->Xdisplay, r->TermWin.gc, GCBackground, &gcvalue);
+	XSetWindowBackground(r->Xdisplay, r->TermWin.vt, r->PixColors[Color_bg]);
+    }
+#if defined(TRANSPARENT) || defined(XPM_BACKGROUND)
+    else if (r->Options & Opt_transparent)
+	rxvt_check_our_parents(r);
+    else if (r->TermWin.pixmap != None)
+	XSetWindowBackgroundPixmap(r->Xdisplay, r->TermWin.vt, r->TermWin.pixmap);
+#endif
+
+    rxvt_scr_clear(r);
+    rxvt_scr_touch(r, True);
+}
+
+
 /*{{{ process an X event */
 /* INTPROTO */
 void
 rxvt_process_x_event(rxvt_t *r, XEvent *ev)
 {
-    int             i, want_time = 0;
+    int             i, want_timeout;
     Window          unused_root, unused_child;
     int             unused_root_x, unused_root_y;
     unsigned int    unused_mask;
@@ -907,21 +1099,19 @@ rxvt_process_x_event(rxvt_t *r, XEvent *ev)
     struct tm      *ltt;
 #endif
 
-#ifdef DEBUG_X
-    want_time = 1;
-#else
     /*
      * check if we need to get the time for any timeouts
      */
 
     for (i = NUM_TIMEOUTS; i--; )
 	if (h->timeout[i].tv_sec) {
-	    want_time = 1;
+	    want_timeout = 1;
 	    break;
 	}
-#endif
 
-    if (want_time)
+#ifndef DEBUG_X
+    if (want_timeout)
+#endif
 	(void)gettimeofday(&tp, NULL);
 
 #ifdef DEBUG_X
@@ -930,7 +1120,7 @@ rxvt_process_x_event(rxvt_t *r, XEvent *ev)
 #endif
 
     /* X event timeouts */
-    if (want_time)
+    if (want_timeout)
 	for (i = NUM_TIMEOUTS; i--; ) {
 	    if (h->timeout[i].tv_sec == 0)
 		continue;
@@ -1043,6 +1233,7 @@ rxvt_process_x_event(rxvt_t *r, XEvent *ev)
 	if (!r->TermWin.focus) {
 	    r->TermWin.focus = 1;
 	    h->want_refresh = 1;
+	    rxvt_set_bg_focused(r, True);
 #ifdef USE_XIM
 	    if (h->Input_Context != NULL)
 		XSetICFocus(h->Input_Context);
@@ -1054,6 +1245,7 @@ rxvt_process_x_event(rxvt_t *r, XEvent *ev)
 	if (r->TermWin.focus) {
 	    r->TermWin.focus = 0;
 	    h->want_refresh = 1;
+	    rxvt_set_bg_focused(r, False);
 #ifdef USE_XIM
 	    if (h->Input_Context != NULL)
 		XUnsetICFocus(h->Input_Context);
@@ -1068,11 +1260,19 @@ rxvt_process_x_event(rxvt_t *r, XEvent *ev)
 	    do {	/* Wrap lots of configures into one */
 		width = ev->xconfigure.width;
 		height = ev->xconfigure.height;
+		D_SIZE((stderr, "Size: ConfigureNotify: %4d x %4d", width, height));
 	    } while (XCheckTypedWindowEvent(r->Xdisplay, ev->xconfigure.window,
 					    ConfigureNotify, ev));
-	    if (r->szHint.width != width || r->szHint.height != height)
+	    if (r->szHint.width != width || r->szHint.height != height) {
+		D_SIZE((stderr, "Size: Resizing from: %4d x %4d", r->szHint.width, r->szHint.height));
 		rxvt_resize_all_windows(r, (unsigned int)width,
 					(unsigned int)height, 1);
+	    }
+#ifdef DEBUG_SIZE
+	    else {
+		D_SIZE((stderr, "Size: Not resizing"));
+	    }
+#endif
 #ifdef TRANSPARENT		/* XXX: maybe not needed - leave in for now */
 	    if (r->Options & Opt_transparent) {
 		rxvt_check_our_parents(r);
@@ -1162,6 +1362,10 @@ rxvt_process_x_event(rxvt_t *r, XEvent *ev)
 	break;
 
     case MotionNotify:
+#ifdef POINTER_BLANK
+	if (r->h->hidden_pointer)
+	    rxvt_pointer_unblank(r);
+#endif
 	if (isMenuBarWindow(ev->xany.window)) {
 	    rxvt_menubar_control(r, &(ev->xbutton));
 	    break;
@@ -1180,10 +1384,56 @@ rxvt_process_x_event(rxvt_t *r, XEvent *ev)
 			      &unused_mask);
 #ifdef MOUSE_THRESHOLD
 	    /* deal with a `jumpy' mouse */
-		if ((ev->xmotion.time - h->MEvent.time) > MOUSE_THRESHOLD)
+		if ((ev->xmotion.time - h->MEvent.time) > MOUSE_THRESHOLD) {
 #endif
 		    rxvt_selection_extend(r, (ev->xbutton.x), (ev->xbutton.y),
 				  (ev->xbutton.state & Button3Mask) ? 2 : 0);
+#ifdef SELECTION_SCROLLING
+		    if (ev->xbutton.y<r->TermWin.int_bwidth ||
+			Pixel2Row(ev->xbutton.y)>(r->TermWin.nrow-1)) {
+			int dist;
+			
+			h->pending_scroll_selection=1;
+			
+			  /* don't clobber the current delay if we are
+			   * already in the middle of scrolling.
+			   */
+			if (h->scroll_selection_delay<=0)
+			    h->scroll_selection_delay=SCROLLBAR_CONTINUOUS_DELAY;
+
+			  /* save the event params so we can highlight
+			   * the selection in the pending-scroll loop
+			   */
+			h->selection_save_x=ev->xbutton.x;
+			h->selection_save_y=ev->xbutton.y;
+			h->selection_save_state=
+			    (ev->xbutton.state & Button3Mask) ? 2 : 0;
+
+			  /* calc number of lines to scroll */
+			if (ev->xbutton.y<r->TermWin.int_bwidth) {
+			    h->scroll_selection_dir = UP;
+			    dist = r->TermWin.int_bwidth - ev->xbutton.y;
+			}
+			else {
+			    h->scroll_selection_dir = DN;
+			    dist = ev->xbutton.y -
+				(r->TermWin.int_bwidth + r->TermWin.height);
+			}
+			h->scroll_selection_lines=(Pixel2Height(dist)/
+			    SELECTION_SCROLL_LINE_SPEEDUP)+1;
+			MIN_IT(h->scroll_selection_lines,
+			    SELECTION_SCROLL_MAX_LINES);
+		    }
+		    else {
+			  /* we are within the text window, so we
+			   * shouldn't be scrolling
+			   */
+			h->pending_scroll_selection = 0;
+		    }
+#endif
+#ifdef MOUSE_THRESHOLD
+		}
+#endif
 	    }
 	} else if (isScrollbarWindow(ev->xany.window) && scrollbar_isMotion()) {
 	    while (XCheckTypedWindowEvent(r->Xdisplay, r->scrollBar.win,
@@ -1424,6 +1674,9 @@ rxvt_button_release(rxvt_t *r, XButtonEvent *ev)
 	r->h->refresh_type &= ~SMOOTH_REFRESH;
 #endif
     }
+#ifdef SELECTION_SCROLLING
+    r->h->pending_scroll_selection=0;
+#endif
     if (ev->window == r->TermWin.vt) {
 	if (ev->subwindow != None)
 	    rxvt_Gr_ButtonRelease(ev->x, ev->y);
@@ -1473,9 +1726,14 @@ rxvt_button_release(rxvt_t *r, XButtonEvent *ev)
 		    int             i, v;
 
 		    v = (ev->button == Button4) ? UP : DN;
-		    i = (ev->state & ShiftMask) ? 1 : 5;
+		    if (ev->state & ShiftMask)
+			i = 1;
+		    else if ((r->Options & Opt_mouseWheelScrollPage))
+			i = r->TermWin.nrow - 1;
+		    else
+			i = 5;
 # ifdef MOUSE_SLIP_WHEELING
-		    if (ev->state & ControlMask) {
+                    if (ev->state & ControlMask) {
 			r->h->mouse_slip_wheel_speed += (v ? -1 : 1);
 			r->h->mouse_slip_wheel_delay = SCROLLBAR_CONTINUOUS_DELAY;
 		    }
@@ -1778,6 +2036,109 @@ enum {
 };
 /* *INDENT-ON* */
 
+/*{{{ process non-printing single characters */
+/* INTPROTO */
+void
+rxvt_process_nonprinting(rxvt_t *r, unsigned char ch)
+{
+    switch (ch) {
+    case C0_ENQ:	/* terminal Status */
+	if (r->h->rs[Rs_answerbackstring])
+	    rxvt_tt_write(r,
+		(const unsigned char *)r->h->rs[Rs_answerbackstring],
+		(unsigned int)STRLEN(r->h->rs[Rs_answerbackstring]));
+	else
+	    rxvt_tt_write(r, (unsigned char *)VT100_ANS,
+			  (unsigned int)STRLEN(VT100_ANS));
+	break;
+    case C0_BEL:	/* bell */
+	rxvt_scr_bell(r);
+	break;
+    case C0_BS:		/* backspace */
+	rxvt_scr_backspace(r);
+	break;
+    case C0_HT:		/* tab */
+	rxvt_scr_tab(r, 1);
+	break;
+    case C0_CR:		/* carriage return */
+	rxvt_scr_gotorc(r, 0, 0, R_RELATIVE);
+	break;
+    case C0_VT:		/* vertical tab, form feed */
+    case C0_FF:
+    case C0_LF:		/* line feed */
+	rxvt_scr_index(r, UP);
+	break;
+    case C0_SO:		/* shift out - acs */
+	rxvt_scr_charset_choose(r, 1);
+	break;
+    case C0_SI:		/* shift in - acs */
+	rxvt_scr_charset_choose(r, 0);
+	break;
+    }
+}
+/*}}} */
+
+
+/*{{{ process VT52 escape sequences */
+/* INTPROTO */
+void
+rxvt_process_escape_vt52(rxvt_t *r, unsigned char ch)
+{
+    int row, col;
+    
+    switch (ch) {
+    case 'A':		/* cursor up */
+	rxvt_scr_gotorc(r, -1, 0, R_RELATIVE | C_RELATIVE);	
+	break;
+    case 'B':		/* cursor down */
+	rxvt_scr_gotorc(r, 1, 0, R_RELATIVE | C_RELATIVE);	
+	break;
+    case 'C':		/* cursor right */
+	rxvt_scr_gotorc(r, 0, 1, R_RELATIVE | C_RELATIVE);	
+	break;
+    case 'D':		/* cursor left */
+	rxvt_scr_gotorc(r, 0, -1, R_RELATIVE | C_RELATIVE);	
+	break;
+    case 'H':		/* cursor home */
+	rxvt_scr_gotorc(r, 0, 0, 0);	
+	break;
+    case 'I':		/* cursor up and scroll down if needed */
+	rxvt_scr_index(r, DN);
+	break;
+    case 'J':		/* erase to end of screen */
+	rxvt_scr_erase_screen(r, 0);
+	break;
+    case 'K':		/* erase to end of line */
+	rxvt_scr_erase_line(r, 0);
+	break;
+    case 'Y':         	/* move to specified row and col */
+	  /* full command is 'ESC Y row col' where row and col
+	   * are encoded by adding 32 and sending the ascii
+	   * character.  eg. SPACE = 0, '+' = 13, '0' = 18,
+	   * etc. */
+	row = rxvt_cmd_getc(r) - ' ';
+	col = rxvt_cmd_getc(r) - ' ';
+	rxvt_scr_gotorc(r, row, col, 0);
+	break;
+    case 'Z':		/* identify the terminal type */
+	rxvt_tt_printf(r, "\033/Z");	/* I am a VT100 emulating a VT52 */
+        break;
+    case '<':		/* turn off VT52 mode */
+        PrivMode(0, PrivMode_vt52);
+	break;
+    case 'F':     	/* use special graphics character set */
+    case 'G':           /* use regular character set */
+	  /* unimplemented */
+	break;
+    case '=':     	/* use alternate keypad mode */
+    case '>':           /* use regular keypad mode */
+	  /* unimplemented */
+	break;
+    }
+}
+/*}}} */
+
+
 /*{{{ process escape sequences */
 /* INTPROTO */
 void
@@ -1785,6 +2146,11 @@ rxvt_process_escape_seq(rxvt_t *r)
 {
     unsigned char   ch = rxvt_cmd_getc(r);
 
+    if (r->h->PrivateModes & PrivMode_vt52) {
+	rxvt_process_escape_vt52(r, ch);
+	return;
+    }
+    
     switch (ch) {
     /* case 1:        do_tek_mode (); break; */
     case '#':
@@ -1969,8 +2335,7 @@ rxvt_process_csi_seq(rxvt_t *r)
 	    rxvt_process_escape_seq(r);
 	    return;
 	} else if (ch < ' ') {
-	    rxvt_scr_add_lines(r, &ch, 0, 1);
-	    return;
+	    rxvt_process_nonprinting(r, ch);
 	}
 	ch = rxvt_cmd_getc(r);
     }
@@ -2118,11 +2483,8 @@ rxvt_process_csi_seq(rxvt_t *r)
 	break;
 
     case CSI_DA:		/* 8.3.24: (0) DEVICE ATTRIBUTES */
-	if (priv == '>')	/* secondary device attributes */
-	    rxvt_tt_printf(r, "\033[>%d;%-.8s;0c", 'R', VSTRING);
-	else if (!priv && arg[0] == 0)
-	    rxvt_tt_write(r, (const unsigned char *)VT100_ANS,
-			  (unsigned int)(sizeof(VT100_ANS) - 1));
+	rxvt_tt_write(r, (const unsigned char *)VT100_ANS,
+		      (unsigned int)(sizeof(VT100_ANS) - 1));
 	break;
 
     case CSI_SGR:		/* 8.3.118: (0) SELECT GRAPHIC RENDITION */
@@ -2234,7 +2596,9 @@ void
 rxvt_process_window_ops(rxvt_t *r, const int *args, unsigned int nargs)
 {
     int             x, y;
+#if 0
     char           *s;
+#endif
     XWindowAttributes wattr;
     Window          wdummy;
 
@@ -2296,6 +2660,7 @@ rxvt_process_window_ops(rxvt_t *r, const int *args, unsigned int nargs)
     case 18:			/* report window size (chars) */
 	rxvt_tt_printf(r, "\033[8;%d;%dt", r->TermWin.nrow, r->TermWin.ncol);
 	break;
+#if 0 /* XXX: currently disabled due to security concerns */
     case 20:			/* report icon label */
 	XGetIconName(r->Xdisplay, r->TermWin.parent[0], &s);
 	rxvt_tt_printf(r, "\033]L%-.200s\234", s ? s : "");	/* 8bit ST */
@@ -2304,6 +2669,7 @@ rxvt_process_window_ops(rxvt_t *r, const int *args, unsigned int nargs)
 	XFetchName(r->Xdisplay, r->TermWin.parent[0], &s);
 	rxvt_tt_printf(r, "\033]l%-.200s\234", s ? s : "");	/* 8bit ST */
 	break;
+#endif
     }
 }
 #endif
@@ -2387,7 +2753,11 @@ rxvt_process_osc_seq(rxvt_t *r)
      * so do it here
      */
 	    if (arg == XTerm_Menu)
+#if 0 /* XXX: currently disabled due to security concerns */
 		rxvt_menubar_dispatch(r, (char *)s);
+#else
+		0;
+#endif
 	    else
 		rxvt_xterm_seq(r, arg, (char *)s, eh);
 	    free(s);
@@ -2420,7 +2790,6 @@ void
 rxvt_xterm_seq(rxvt_t *r, int op, const char *str, unsigned char resp __attribute__((unused)))
 {
     int             changed = 0;
-    int             fd;
     int             color;
     char           *buf, *name;
 
@@ -2501,12 +2870,17 @@ rxvt_xterm_seq(rxvt_t *r, int op, const char *str, unsigned char resp __attribut
     case XTerm_font:
 	rxvt_change_font(r, 0, str);
 	break;
+#if 0
     case XTerm_dumpscreen:	/* no error notices */
-	if ((fd = open(str, O_RDWR | O_CREAT | O_EXCL, 0600)) >= 0) {
-	    rxvt_scr_dump(r, fd);
-	    close(fd);
+	{
+    	    int             fd;
+	    if ((fd = open(str, O_RDWR | O_CREAT | O_EXCL, 0600)) >= 0) {
+		rxvt_scr_dump(r, fd);
+		close(fd);
+	    }
 	}
 	break;
+#endif
     }
 }
 /*----------------------------------------------------------------------*/
@@ -2552,6 +2926,7 @@ rxvt_process_terminal_mode(rxvt_t *r, int mode, int priv __attribute__((unused))
 	const unsigned long bit;
     } argtopriv[] = {
 	{ 1, PrivMode_aplCUR },
+	{ 2, PrivMode_vt52 },
 	{ 3, PrivMode_132 },
 	{ 4, PrivMode_smoothScroll },
 	{ 5, PrivMode_rVideo },
@@ -2613,7 +2988,13 @@ rxvt_process_terminal_mode(rxvt_t *r, int mode, int priv __attribute__((unused))
 	/* extra handling for values with valid 0 or 1 state */
 	switch (arg[i]) {
 	/* case 1:	- application cursor keys */
-	/* case 2:	- reset charsets to USASCII */
+	case 2:			/* VT52 mode */
+	      /* oddball mode.  should be set regardless of set/reset
+	       * parameter.  Return from VT52 mode with an ESC < from
+	       * within VT52 mode
+	       */
+	    PrivMode(1, PrivMode_vt52);
+	    break;
 	case 3:			/* 80/132 */
 	    if (r->h->PrivateModes & PrivMode_132OK)
 		rxvt_set_widthheight(r,
@@ -2895,12 +3276,10 @@ rxvt_main_loop(rxvt_t *r)
 
     h->cmdbuf_ptr = h->cmdbuf_endp = h->cmdbuf_base;
 
-#if defined(__CYGWIN32__)
     /* once we know the shell is running, send the screen size.  Again! */
     (void)rxvt_cmd_getc(r);	/* wait for something */
     h->cmdbuf_ptr--;		/* unget it - reprocess it below */
-    rxvt_tt_winsize(r->cmd_fd, r->TermWin.ncol, r->TermWin.nrow);
-#endif
+    rxvt_tt_winsize(r->cmd_fd, r->TermWin.ncol, r->TermWin.nrow, r->h->cmd_pid);
 
     refreshnow = 0;
     for (;;) {
@@ -2949,30 +3328,8 @@ rxvt_main_loop(rxvt_t *r)
 	    }
 	} else
 	    switch (ch) {
-	    case C0_ENQ:	/* terminal Status */
-		if (h->rs[Rs_answerbackstring])
-		    rxvt_tt_write(r,
-			(const unsigned char *)h->rs[Rs_answerbackstring],
-			(unsigned int)STRLEN(h->rs[Rs_answerbackstring]));
-		else
-		    rxvt_tt_write(r, (unsigned char *)VT100_ANS,
-				  (unsigned int)STRLEN(VT100_ANS));
-		break;
-	    case C0_BEL:	/* bell */
-		rxvt_scr_bell(r);
-		break;
-	    case C0_BS:		/* backspace */
-		rxvt_scr_backspace(r);
-		break;
-	    case C0_VT:		/* vertical tab, form feed */
-	    case C0_FF:
-		rxvt_scr_index(r, UP);
-		break;
-	    case C0_SO:		/* shift out - acs */
-		rxvt_scr_charset_choose(r, 1);
-		break;
-	    case C0_SI:		/* shift in - acs */
-		rxvt_scr_charset_choose(r, 0);
+	    default:
+		rxvt_process_nonprinting(r, ch);
 		break;
 	    case C0_ESC:	/* escape char */
 		rxvt_process_escape_seq(r);
