@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	main.c
  *----------------------------------------------------------------------*
- * $Id: main.c,v 1.173 2003/03/28 01:41:11 gcw Exp $
+ * $Id: main.c,v 1.179 2004/01/29 23:54:50 gcw Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (c) 1992      John Bovey, University of Kent at Canterbury <jdb@ukc.ac.uk>
@@ -396,6 +396,7 @@ rxvt_window_calc(rxvt_t *r, unsigned int width, unsigned int height)
     unsigned int    w, h;
     unsigned int    max_width, max_height;
 
+    D_SIZE((stderr, "< Cols/Rows: %3d x %3d ; Width/Height: %4d x %4d", r->TermWin.ncol, r->TermWin.nrow, r->szHint.width, r->szHint.height));
     r->szHint.flags = PMinSize | PResizeInc | PBaseSize | PWinGravity;
     r->szHint.win_gravity = NorthWestGravity;
     /* r->szHint.min_aspect.x = r->szHint.min_aspect.y = 1; */
@@ -486,6 +487,7 @@ rxvt_window_calc(rxvt_t *r, unsigned int width, unsigned int height)
 
     r->TermWin.ncol = r->TermWin.width / r->TermWin.fwidth;
     r->TermWin.nrow = r->TermWin.height / r->TermWin.fheight;
+    D_SIZE((stderr, "> Cols/Rows: %3d x %3d ; Width/Height: %4d x %4d", r->TermWin.ncol, r->TermWin.nrow, r->szHint.width, r->szHint.height));
     return;
 }
 
@@ -496,7 +498,7 @@ rxvt_window_calc(rxvt_t *r, unsigned int width, unsigned int height)
  */
 /* EXTPROTO */
 void
-rxvt_tt_winsize(int fd, unsigned short col, unsigned short row)
+rxvt_tt_winsize(int fd, unsigned short col, unsigned short row, int pid)
 {
     struct winsize  ws;
 
@@ -505,7 +507,17 @@ rxvt_tt_winsize(int fd, unsigned short col, unsigned short row)
     ws.ws_col = col;
     ws.ws_row = row;
     ws.ws_xpixel = ws.ws_ypixel = 0;
-    ioctl(fd, TIOCSWINSZ, &ws);
+#ifndef DEBUG_SIZE
+    (void)ioctl(fd, TIOCSWINSZ, &ws);
+#else
+    if (ioctl(fd, TIOCSWINSZ, &ws) < 0) {
+	D_SIZE((stderr, "Failed to send TIOCSWINSZ to fd %d", fd));
+    }
+# ifdef SIGWINCH
+    else if (pid)		/* force through to the command */
+	kill(pid, SIGWINCH);
+# endif
+#endif
 }
 
 /*----------------------------------------------------------------------*/
@@ -688,7 +700,7 @@ rxvt_change_font(rxvt_t *r, int init, const char *fontname)
 	    c = "-*-%.2d-*-jisx0208*-*";
 	    enc = "EUCJ/SJIS";
 	    break;
-	case EUCKR:
+	case KR:
 	    c = "-*-%.2d-*-ksc5601*-*";
 	    enc = "KR";
 	    break;
@@ -696,8 +708,8 @@ rxvt_change_font(rxvt_t *r, int init, const char *fontname)
 	    i = fh;		/* jump past next two sections */
 	    break;
 	}
-	for (; i < fh / 2; i++) {
-	    sprintf(tmpbuf, c, fh - i);
+	for (; i < (fh - r->TermWin.lineSpace) / 2; i++) {
+	    sprintf(tmpbuf, c, fh - i - r->TermWin.lineSpace);
 	    xfont = XLoadQueryFont(r->Xdisplay, tmpbuf);
 	    if (xfont) {
 		r->h->rs[Rs_mfont + idx] = rxvt_malloc(STRLEN(tmpbuf) + 1);
@@ -879,9 +891,12 @@ rxvt_set_window_color(rxvt_t *r, int idx, const char *color)
 /* XSetWindowAttributes attr; */
 /* Cursor cursor; */
   Done:
-    if (idx == Color_bg && !(r->Options & Opt_transparent))
-	XSetWindowBackground(r->Xdisplay, r->TermWin.vt,
-			     r->PixColors[Color_bg]);
+    if (idx == Color_bg) {
+	r->h->color_origbg = r->PixColors[Color_bg];
+	if (!(r->Options & Opt_transparent))
+	    XSetWindowBackground(r->Xdisplay, r->TermWin.vt,
+				 r->PixColors[Color_bg]);
+    }
 
 /* handle Color_BD, scrollbar background, etc. */
 
