@@ -1,5 +1,5 @@
 /*
- * $Id: rxvt.h,v 1.159 2002/10/01 04:32:40 gcw Exp $
+ * $Id: rxvt.h,v 1.173 2004/01/29 23:54:19 gcw Exp $
  */
 
 #ifndef _RXVT_H_		/* include once only */
@@ -324,7 +324,7 @@ enum {
 
 #define RS_None			0	/* Normal */
 
-#if defined(TTY_256COLOR) || defined(MULTICHAR_SET)
+#if defined(TTY_256COLOR)
 /* have at least 32 bits to use */
 # define RS_fgMask		0x000001FFu	/* 512 colors */
 # define RS_bgMask		0x0003FE00u	/* 512 colors */
@@ -410,7 +410,7 @@ enum {
 /* Words starting with `Color_' are colours.  Others are counts */
 /*
  * The following comment is mostly obsolete since pixcolor_set was expanded:
- * We're currently upto 29 colours.  Only 3 more available.  The
+ * We're currently upto 30 colours.  Only 2 more available.  The
  * PixColor and rendition colour usage should probably be decoupled
  * on the unnecessary items, e.g. Color_pointer, but won't bother
  * until we need to.  Also, be aware of usage in pixcolor_set
@@ -453,6 +453,7 @@ enum colour_list {
 #endif
     Color_pointer,
     Color_border,
+    Color_ufbg,
 #ifndef NO_BOLD_UNDERLINE_REVERSE
     Color_BD,
     Color_UL,
@@ -469,7 +470,7 @@ enum colour_list {
 #ifdef KEEP_SCROLLCOLOR
     Color_topShadow = NRS_COLORS,
     Color_bottomShadow,
-    TOTAL_COLORS		/* upto 30 */
+    TOTAL_COLORS		/* upto 31 */
 #else
     TOTAL_COLORS = NRS_COLORS	/* */
 #endif
@@ -479,11 +480,7 @@ enum colour_list {
 # define Color_Bits	9
 # define NPIXCLR_SETS	9	/* (256 + 14) bits / 32 bits */
 #else
-# ifdef MULTICHAR_SET
-#  define Color_Bits     9
-# else
-#  define Color_Bits	5
-# endif
+# define Color_Bits	5
 # define NPIXCLR_SETS	1	/* (16 + 14) bits / 32 bits */
 #endif
 #define NPIXCLR_BITS	32
@@ -548,6 +545,9 @@ enum {
 #ifdef META8_OPTION
     Rs_meta8,
 #endif
+#ifdef MOUSE_WHEEL
+    Rs_mouseWheelScrollPage,
+#endif
 #ifndef NO_BACKSPACE_KEY
     Rs_backspace_key,
 #endif
@@ -579,9 +579,15 @@ enum {
     Rs_lineSpace,
 #endif
     Rs_cutchars,
+#ifdef ACS_ASCII
+    Rs_acs_chars,
+#endif
     Rs_modifier,
     Rs_answerbackstring,
     Rs_tripleclickwords,
+    Rs_cursorBlink,
+    Rs_pointerBlank,
+    Rs_pointerBlankDelay,
     NUM_RESOURCES
 } ;
 
@@ -637,6 +643,7 @@ enum {
 #define PrivMode_TtyOutputInh	(1LU<<16)
 #define PrivMode_Keypress	(1LU<<17)
 #define PrivMode_smoothScroll	(1LU<<18)
+#define PrivMode_vt52		(1LU<<19)
 /* too annoying to implement X11 highlight tracking */
 /* #define PrivMode_MouseX11Track       (1LU<<18) */
 
@@ -846,6 +853,11 @@ enum {
 #else
 # define D_SELECT(x)
 #endif
+#ifdef DEBUG_SIZE
+# define D_SIZE(x)		fprintf x ; fputc('\n', stderr)
+#else
+# define D_SIZE(x)
+#endif
 #ifdef DEBUG_X
 # define D_X(x)			fprintf x ; fputc('\n', stderr)
 #else
@@ -874,6 +886,8 @@ struct rxvt_hidden {
     unsigned char   chstat:1,
                     lost_multi:1,
                     multi_byte:1,
+                    hidden_cursor:1,
+                    hidden_pointer:1,
                     parsed_geometry:1;
 #else
     unsigned char   want_refresh,
@@ -892,6 +906,12 @@ struct rxvt_hidden {
                     chstat,
                     lost_multi,	/* set ==> we only got half a glyph */
                     multi_byte,	/* set ==> currently using 2 bytes per glyph */
+# endif
+# ifdef CURSOR_BLINK
+                    hidden_cursor,
+# endif
+# ifdef POINTER_BLANK
+                    hidden_pointer,
 # endif
                     parsed_geometry;
 #endif	/* !__GNUC__ */
@@ -924,6 +944,16 @@ struct rxvt_hidden {
     rend_t          rstyle;
 /* ---------- */
     u_int32_t       pixcolor_set[NPIXCLR_SETS];
+/* ---------- */
+#ifdef SELECTION_SCROLLING
+    int		    scroll_selection_delay,
+		    scroll_selection_lines;
+    enum page_dirn  scroll_selection_dir;
+    int		    selection_save_x,
+		    selection_save_y,
+		    selection_save_state,
+		    pending_scroll_selection;
+#endif
 /* ---------- */
     int             csrO,	/* Hops - csr offset in thumb/slider to      */
 				/*   give proper Scroll behaviour            */
@@ -952,6 +982,9 @@ struct rxvt_hidden {
                     window_vt_x,
                     window_vt_y,
                     window_sb_x,
+# ifdef POINTER_BLANK
+		    pointerBlankDelay,
+# endif
                     allowedxerror;
 /* ---------- */
     unsigned int    ModMetaMask,
@@ -964,6 +997,8 @@ struct rxvt_hidden {
                     ttymode;
     unsigned long   PrivateModes,
                     SavedModes;
+/* ---------- */
+    Pixel           color_origbg;
 /* ---------- */
 #ifdef PREFER_24BIT
     Visual         *Xvisual;
@@ -996,8 +1031,8 @@ struct rxvt_hidden {
                     downArrowHi;
 #endif
 /* ---------- */
-    Time            selection_time;
-    Time            selection_request_time;
+    Time            selection_time,
+                    selection_request_time;
     pid_t           cmd_pid;	/* process id of child */
     gid_t           ttygid;
 #if (defined(HAVE_SETEUID) || defined(HAVE_SETREUID)) && !defined(__CYGWIN32__)
@@ -1005,7 +1040,10 @@ struct rxvt_hidden {
     gid_t           egid;
 #endif
 /* ---------- */
-    Cursor          cursor_leftptr;
+    Cursor          pointer_leftptr;
+#ifdef POINTER_BLANK
+    Cursor          pointer_blank;
+#endif
 /* ---------- */
     const char     *ttydev;	/* pty/tty name */
 #ifndef NO_BACKSPACE_KEY
@@ -1038,6 +1076,8 @@ struct rxvt_hidden {
 # endif
 # ifdef HAVE_STRUCT_UTMPX
     struct utmpx    utx;
+# endif
+# if (defined(HAVE_STRUCT_UTMP) && defined(HAVE_UTMP_PID)) || defined(HAVE_STRUCT_UTMPX)
     char            ut_id[5];
 # endif
     int             utmp_pos;
@@ -1064,6 +1104,12 @@ struct rxvt_hidden {
 #endif
 #ifdef RXVT_GRAPHICS
     Window          gr_last_id;
+#endif
+#ifdef CURSOR_BLINK
+    struct timeval  lastcursorchange;
+#endif
+#ifdef POINTER_BLANK
+    struct timeval  lastmotion;
 #endif
     struct timeval  timeout[NUM_TIMEOUTS];
 
